/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(convolutional_encoder.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(a6495f8aaff7b500ea1b88467f005622)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/convolutional_encoder.h>
// pydoc.h is automatically generated in the build directory
#include <convolutional_encoder_pydoc.h>

void bind_convolutional_encoder(py::module& m)
{

    using convolutional_encoder = ::gr::satellites::convolutional_encoder;


    py::class_<convolutional_encoder,
               gr::block,
               gr::basic_block,
               std::shared_ptr<convolutional_encoder>>(
        m, "convolutional_encoder", D(convolutional_encoder))

        .def(py::init(&convolutional_encoder::make),
             py::arg("constraint"),
             py::arg("polynomials"),
             D(convolutional_encoder, make))


        ;
}
