/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(matrix_deinterleaver_soft.h) */
/* BINDTOOL_HEADER_FILE_HASH(ff360136b96f6fe6a1b3db61c269098e)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/matrix_deinterleaver_soft.h>
// pydoc.h is automatically generated in the build directory
#include <matrix_deinterleaver_soft_pydoc.h>

void bind_matrix_deinterleaver_soft(py::module& m)
{

    using matrix_deinterleaver_soft = ::gr::satellites::matrix_deinterleaver_soft;


    py::class_<matrix_deinterleaver_soft,
               gr::block,
               gr::basic_block,
               std::shared_ptr<matrix_deinterleaver_soft>>(
        m, "matrix_deinterleaver_soft", D(matrix_deinterleaver_soft))

        .def(py::init(&matrix_deinterleaver_soft::make),
             py::arg("rows"),
             py::arg("cols"),
             py::arg("output_size"),
             py::arg("output_skip"),
             D(matrix_deinterleaver_soft, make))


        ;
}
