/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(phase_unwrap.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(d2f72c442c14f500648799c8bb4b06c8)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/phase_unwrap.h>
// pydoc.h is automatically generated in the build directory
#include <phase_unwrap_pydoc.h>

void bind_phase_unwrap(py::module& m)
{

    using phase_unwrap = ::gr::satellites::phase_unwrap;


    py::class_<phase_unwrap,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<phase_unwrap>>(m, "phase_unwrap", D(phase_unwrap))

        .def(py::init(&phase_unwrap::make), D(phase_unwrap, make))


        ;
}
