/* -*- c++ -*- */
/*
 * Copyright 2018 Daniel Estevez <daniel@destevez.net>
 *
 * This file is part of gr-satellites
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

#ifndef _RS_H
#define _RS_H

#define MAX_FRAME_LEN 255
#define PARITY_BYTES 32

#define BASIS_CONVENTIONAL 0
#define BASIS_DUAL 1

#endif
