/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(u482c_decode.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(0be7889ffa99c1ff24f997fa4119bc5b)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/u482c_decode.h>
// pydoc.h is automatically generated in the build directory
#include <u482c_decode_pydoc.h>

void bind_u482c_decode(py::module& m)
{

    using u482c_decode = ::gr::satellites::u482c_decode;


    py::class_<u482c_decode, gr::block, gr::basic_block, std::shared_ptr<u482c_decode>>(
        m, "u482c_decode", D(u482c_decode))

        .def(py::init(&u482c_decode::make),
             py::arg("verbose"),
             py::arg("viterbi"),
             py::arg("scrambler"),
             py::arg("rs"),
             D(u482c_decode, make))


        ;
}
