/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(varlen_packet_tagger.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(0ece53d0fe74c0301bf92e4ee6d6b39d)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/varlen_packet_tagger.h>
// pydoc.h is automatically generated in the build directory
#include <varlen_packet_tagger_pydoc.h>

void bind_varlen_packet_tagger(py::module& m)
{

    using varlen_packet_tagger = ::gr::satellites::varlen_packet_tagger;


    py::class_<varlen_packet_tagger,
               gr::block,
               gr::basic_block,
               std::shared_ptr<varlen_packet_tagger>>(
        m, "varlen_packet_tagger", D(varlen_packet_tagger))

        .def(py::init(&varlen_packet_tagger::make),
             py::arg("sync_key"),
             py::arg("packet_key"),
             py::arg("length_field_size"),
             py::arg("max_packet_size"),
             py::arg("endianness"),
             py::arg("use_golay"),
             D(varlen_packet_tagger, make))


        ;
}
