/*
 * Copyright 2023 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(doppler_correction.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(56444490a7f522137e132976be778b10)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/doppler_correction.h>
// pydoc.h is automatically generated in the build directory
#include <doppler_correction_pydoc.h>

void bind_doppler_correction(py::module& m)
{

    using doppler_correction = ::gr::satellites::doppler_correction;


    py::class_<doppler_correction,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<doppler_correction>>(
        m, "doppler_correction", D(doppler_correction))

        .def(py::init(&doppler_correction::make),
             py::arg("filename"),
             py::arg("samp_rate"),
             py::arg("t0"),
             D(doppler_correction, make))


        .def("set_time",
             &doppler_correction::set_time,
             py::arg("t"),
             D(doppler_correction, set_time))


        .def("time", &doppler_correction::time, D(doppler_correction, time))


        .def(
            "frequency", &doppler_correction::frequency, D(doppler_correction, frequency))

        ;
}
