/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(pdu_scrambler.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(ec8b3adb18a42c4389fc879f7fc50ee4)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/pdu_scrambler.h>
// pydoc.h is automatically generated in the build directory
#include <pdu_scrambler_pydoc.h>

void bind_pdu_scrambler(py::module& m)
{

    using pdu_scrambler = ::gr::satellites::pdu_scrambler;


    py::class_<pdu_scrambler, gr::block, gr::basic_block, std::shared_ptr<pdu_scrambler>>(
        m, "pdu_scrambler", D(pdu_scrambler))

        .def(py::init(&pdu_scrambler::make), py::arg("sequence"), D(pdu_scrambler, make))


        ;
}
