/*
 * Copyright 2022 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(fixedlen_to_pdu.h)                                        */
/* BINDTOOL_HEADER_FILE_HASH(ee51137386e71610d87ab3bd1c5ccbf2)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <satellites/fixedlen_to_pdu.h>
// pydoc.h is automatically generated in the build directory
#include <fixedlen_to_pdu_pydoc.h>

void bind_fixedlen_to_pdu(py::module& m)
{

    using fixedlen_to_pdu = ::gr::satellites::fixedlen_to_pdu;


    py::class_<fixedlen_to_pdu,
               gr::sync_block,
               gr::block,
               gr::basic_block,
               std::shared_ptr<fixedlen_to_pdu>>(m, "fixedlen_to_pdu", D(fixedlen_to_pdu))

        .def(py::init(&fixedlen_to_pdu::make),
             py::arg("type"),
             py::arg("syncword_tag"),
             py::arg("packet_len"),
             py::arg("pack") = false,
             D(fixedlen_to_pdu, make))


        ;
}
